@@ -214,22 +214,28 @@ query_callback (gpointer inthread_data, GError *error, gpointer user_data)
 	InThreadPtr *ptr = inthread_data;
 	TrackerDBusMethodInfo *info = user_data;
 
-	if (ptr->error) {
+	if (ptr && ptr->error) {
 		tracker_dbus_request_failed (info->request_id,
 		                             info->context,
 		                             &ptr->error,
 		                             NULL);
 		dbus_g_method_return_error (info->context, ptr->error);
 		g_error_free (ptr->error);
+	} else if (error) {
+		tracker_dbus_request_failed (info->request_id,
+		                             info->context,
+		                             &error,
+		                             NULL);
+		dbus_g_method_return_error (info->context, error);
 	} else {
-
 		tracker_dbus_request_success (info->request_id,
 		                              info->context);
 
 		dbus_g_method_send_reply (info->context, ptr->reply);
 	}
 
-	g_slice_free (InThreadPtr, ptr);
+	if (ptr)
+		g_slice_free (InThreadPtr, ptr);
 }
 
 static gpointer
@@ -277,7 +283,7 @@ query_inthread (TrackerDBCursor *cursor, GError *error, gpointer user_data)
 
 		for (i = 0; i < cols && cont; i++, length++) {
 			const gchar *result_str;
-			result_str = tracker_db_cursor_get_string (cursor, i);
+			result_str = tracker_db_cursor_get_string (cursor, i, NULL);
 
 			if (result_str == NULL)
 				result_str = "";
@@ -317,12 +323,9 @@ tracker_resources_sparql_query (TrackerResources         *self,
                                 GError                  **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (query != NULL, context);
@@ -374,12 +377,9 @@ tracker_resources_sparql_update (TrackerResources        *self,
                                  GError                 **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (update != NULL, context);
@@ -430,12 +430,9 @@ tracker_resources_sparql_update_blank (TrackerResources       *self,
                                        GError                **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (update != NULL, context);
@@ -487,12 +484,9 @@ tracker_resources_batch_sparql_update (TrackerResources          *self,
                                        GError                   **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (update != NULL, context);
@@ -536,12 +530,9 @@ tracker_resources_batch_commit (TrackerResources         *self,
                                 GError                  **error)
 {
 	TrackerDBusMethodInfo *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
@@ -576,6 +567,9 @@ on_statements_committed (gpointer user_data)
 	/* Class signals feature */
 	events = tracker_events_get_pending ();
 
+	/* Do not call tracker_events_reset before calling tracker_resource_class_emit_events
+	   as we're reusing the same strings without copies */
+
 	if (events) {
 		GSList *event_sources, *l;
 		guint i;
@@ -591,7 +585,7 @@ on_statements_committed (gpointer user_data)
 			for (l = event_sources; l; l = l->next) {
 				TrackerResourceClass *class_ = l->data;
 				if (g_strcmp0 (tracker_class_get_uri (event->class), tracker_resource_class_get_rdf_class (class_)) == 0) {
-					tracker_resource_class_add_event (class_, event->subject, tracker_property_get_uri (event->predicate), event->type);
+					tracker_resource_class_add_event (class_, event->subject, event->predicate, event->type);
 					if (!to_emit) {
 						to_emit = g_hash_table_new (NULL, NULL);
 					}
