@@ -21,6 +21,10 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE
+#include <sched.h>
+#include <pthread.h>
+
 #include <unistd.h>
 #include <sys/types.h>
 
@@ -38,6 +42,8 @@
 #define TRACKER_STORE_TRANSACTION_MAX                   4000
 #define TRACKER_STORE_MAX_CONCURRENT_QUERIES               2
 
+#define TRACKER_STORE_N_TURTLE_STATEMENTS                 50
+
 #define TRACKER_STORE_QUERY_WATCHDOG_TIMEOUT 10
 #define TRACKER_STORE_MAX_TASK_TIME          30
 
@@ -53,6 +59,7 @@ typedef struct {
 	GThreadPool *global_pool;
 	GSList	    *running_tasks;
 	guint	     watchdog_id;
+	guint        max_task_time;
 } TrackerStorePrivate;
 
 typedef enum {
@@ -78,8 +85,9 @@ typedef struct {
 			GPtrArray    *blank_nodes;
 		} update;
 		struct {
-			gboolean      in_progress;
-			gchar        *path;
+			TrackerTurtleReader *reader;
+			gboolean             in_progress;
+			gchar               *path;
 		} turtle;
 	} data;
 	gchar                *client_id;
@@ -100,6 +108,11 @@ typedef struct {
 
 static GStaticPrivate private_key = G_STATIC_PRIVATE_INIT;
 
+#ifdef __USE_GNU
+/* cpu used for mainloop thread and main update/query thread */
+static int main_cpu;
+#endif /* __USE_GNU */
+
 static void start_handler (TrackerStorePrivate *private);
 
 static void
@@ -118,6 +131,7 @@ static void
 store_task_free (TrackerStoreTask *task)
 {
 	if (task->type == TRACKER_STORE_TASK_TYPE_TURTLE) {
+		g_object_unref (task->data.turtle.reader);
 		g_free (task->data.turtle.path);
 	} else if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
 		g_free (task->data.query.query);
@@ -173,7 +187,7 @@ process_turtle_file_part (TrackerTurtleReader *reader, GError **error)
 		}
 
 		i++;
-		if (!new_error && i >= 10) {
+		if (!new_error && i >= TRACKER_STORE_N_TURTLE_STATEMENTS) {
 			tracker_data_commit_transaction (&new_error);
 			if (new_error) {
 				tracker_data_rollback_transaction ();
@@ -301,7 +315,8 @@ watchdog_cb (gpointer user_data)
 		running = running->next;
 		thread = task->data.query.running_thread;
 
-		if (thread && g_timer_elapsed (task->data.query.timer, NULL) > TRACKER_STORE_MAX_TASK_TIME) {
+		if (thread && private->max_task_time &&
+		    g_timer_elapsed (task->data.query.timer, NULL) > private->max_task_time) {
 			tracker_data_manager_interrupt_thread (task->data.query.running_thread);
 		}
 	}
@@ -392,6 +407,25 @@ task_finish_cb (gpointer data)
 		}
 
 		private->update_running = FALSE;
+	} else if (task->type == TRACKER_STORE_TASK_TYPE_TURTLE) {
+		private->update_running = FALSE;
+
+		if (task->data.turtle.in_progress) {
+			/* Task still in progress */
+			check_handler (private);
+			return FALSE;
+		} else {
+			if (task->callback.turtle_callback) {
+				task->callback.turtle_callback (task->error, task->user_data);
+			}
+
+			if (task->error) {
+				g_clear_error (&task->error);
+			}
+
+			/* Remove the task now that we're done with it */
+			g_queue_pop_head (private->queues[TRACKER_STORE_PRIORITY_TURTLE]);
+		}
 	} else if (task->type == TRACKER_STORE_TASK_TYPE_COMMIT) {
 		tracker_data_notify_db_transaction ();
 
@@ -419,14 +453,30 @@ pool_dispatch_cb (gpointer data,
 {
 	TrackerStorePrivate *private;
 	TrackerStoreTask *task;
+	GThread *running_thread = g_thread_self ();
+
+#ifdef __USE_GNU
+	/* special task, only ever sent to main pool */
+	if (GPOINTER_TO_INT (data) == 1) {
+		cpu_set_t cpuset;
+		CPU_ZERO (&cpuset);
+		CPU_SET (main_cpu, &cpuset);
+
+		/* avoid cpu hopping which can lead to significantly worse performance */
+		pthread_setaffinity_np (pthread_self (), sizeof (cpu_set_t), &cpuset);
+		return;
+	}
+#endif /* __USE_GNU */
 
 	private = user_data;
 	task = data;
 
+	tracker_data_manager_interrupt_thread_reset (running_thread);
+
 	if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
 		TrackerDBCursor *cursor;
 
-		task->data.query.running_thread = g_thread_self ();
+		task->data.query.running_thread = running_thread;
 		cursor = tracker_data_query_sparql_cursor (task->data.query.query, &task->error);
 
 		task->data.query.thread_data = task->callback.query.in_thread (cursor, task->error, task->user_data);
@@ -475,7 +525,31 @@ pool_dispatch_cb (gpointer data,
 		} else {
 			tracker_data_commit_db_transaction ();
 		}
+	} else if (task->type == TRACKER_STORE_TASK_TYPE_TURTLE) {
+		if (!task->data.turtle.in_progress) {
+			task->data.turtle.reader = tracker_turtle_reader_new (task->data.turtle.path, &task->error);
 
+			if (task->error) {
+				g_idle_add (task_finish_cb, task);
+				return;
+			}
+
+			task->data.turtle.in_progress = TRUE;
+		}
+
+		begin_batch (private);
+
+		if (process_turtle_file_part (task->data.turtle.reader, &task->error)) {
+			/* import still in progress */
+			private->batch_count++;
+			if (private->batch_count >= TRACKER_STORE_TRANSACTION_MAX) {
+				end_batch (private);
+			}
+		} else {
+			/* import finished */
+			task->data.turtle.in_progress = FALSE;
+			end_batch (private);
+		}
 	} else if (task->type == TRACKER_STORE_TASK_TYPE_COMMIT) {
 		end_batch (private);
 	}
@@ -530,71 +604,10 @@ queue_idle_handler (gpointer user_data)
 
 		task_run_async (private, task);
 	} else if (task->type == TRACKER_STORE_TASK_TYPE_TURTLE) {
-		GError *error = NULL;
-		static TrackerTurtleReader *turtle_reader = NULL;
-
-		if (!task->data.turtle.in_progress) {
-			turtle_reader = tracker_turtle_reader_new (task->data.turtle.path, &error);
-			if (error) {
-				if (task->callback.turtle_callback) {
-					task->callback.turtle_callback (error, task->user_data);
-				}
-
-				turtle_reader = NULL;
-				g_clear_error (&error);
-
-				g_queue_pop_head (queue);
-
-				if (task->destroy) {
-					task->destroy (task->user_data);
-				}
-
-				store_task_free (task);
-
-				goto out;
-			}
-			task->data.turtle.in_progress = TRUE;
-		}
-
-		begin_batch (private);
-
-		if (process_turtle_file_part (turtle_reader, &error)) {
-			/* import still in progress */
-			private->batch_count++;
-			if (private->batch_count >= TRACKER_STORE_TRANSACTION_MAX) {
-				end_batch (private);
-			}
-
-			/* Process function wont return true in case of error */
-
-			return TRUE;
-		} else {
-			/* import finished */
-			task->data.turtle.in_progress = FALSE;
-
-			end_batch (private);
-
-			if (task->callback.turtle_callback) {
-				task->callback.turtle_callback (error, task->user_data);
-			}
-
-			g_object_unref (turtle_reader);
-			turtle_reader = NULL;
-			if (error) {
-				g_clear_error (&error);
-			}
-
-			g_queue_pop_head (queue);
-
-			if (task->destroy) {
-				task->destroy (task->user_data);
-			}
-
-			store_task_free (task);
-		}
+		private->update_running = TRUE;
+		task_run_async (private, task);
 	}
 
-out:
 	return task_ready (private);
 }
 
@@ -606,15 +619,24 @@ queue_idle_destroy (gpointer user_data)
 	private->have_handler = FALSE;
 }
 
-
 void
 tracker_store_init (void)
 {
 	TrackerStorePrivate *private;
 	gint i;
+	const char *tmp;
+#ifdef __USE_GNU
+	cpu_set_t cpuset;
+#endif /* __USE_GNU */
 
 	private = g_new0 (TrackerStorePrivate, 1);
 
+	if ((tmp = g_getenv("TRACKER_STORE_MAX_TASK_TIME")) != NULL) {
+		private->max_task_time = atoi (tmp);
+	} else {
+		private->max_task_time = TRACKER_STORE_MAX_TASK_TIME;
+	}
+
 	for (i = 0; i < TRACKER_STORE_N_PRIORITIES; i++) {
 		private->queues[i] = g_queue_new ();
 	}
@@ -632,10 +654,22 @@ tracker_store_init (void)
 	g_thread_pool_set_max_idle_time (15 * 1000);
 	g_thread_pool_set_max_unused_threads (2);
 
+#ifdef __USE_GNU
+	main_cpu = sched_getcpu ();
+	CPU_ZERO (&cpuset);
+	CPU_SET (main_cpu, &cpuset);
+
+	/* avoid cpu hopping which can lead to significantly worse performance */
+	pthread_setaffinity_np (pthread_self (), sizeof (cpu_set_t), &cpuset);
+	/* lock main update/query thread to same cpu to improve overall performance
+	   main loop thread is essentially idle during query execution */
+#endif /* __USE_GNU */
+
+	g_thread_pool_push (private->main_pool, GINT_TO_POINTER (1), NULL);
+
 	g_static_private_set (&private_key,
 	                      private,
 	                      private_free);
-
 }
 
 void
@@ -811,7 +845,7 @@ tracker_store_queue_turtle_import (GFile                      *file,
 	task->callback.update_callback = callback;
 	task->destroy = destroy;
 
-	g_queue_push_tail (private->queues[TRACKER_STORE_PRIORITY_LOW], task);
+	g_queue_push_tail (private->queues[TRACKER_STORE_PRIORITY_TURTLE], task);
 
 	check_handler (private);
 }
@@ -897,4 +931,6 @@ tracker_store_unreg_batches (const gchar *client_id)
 	if (error) {
 		g_clear_error (&error);
 	}
+
+	check_handler (private);
 }
